@@ -1,82 +1,78 @@
-/*  $Revision: 1.1.1.1 $
+/*  $Id: shlock.c 6124 2003-01-14 06:03:29Z rra $
 **
 **  Produce reliable locks for shell scripts, by Peter Honeyman as told
 **  to Rich $alz.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <errno.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <sys/stat.h>
 
+#include "inn/messages.h"
 
-STATIC BOOL	BinaryLock;
-STATIC char	CANTUNLINK[] = "Can't unlink \"%s\", %s\n";
-STATIC char	CANTOPEN[] = "Can't open \"%s\", %s\n";
+
+static bool	BinaryLock;
 
 
 /*
 **  See if the process named in an existing lock still exists by
 **  sending it a null signal.
 */
-STATIC BOOL
-ValidLock(name, JustChecking)
-    char		*name;
-    BOOL		JustChecking;
+static bool
+ValidLock(char *name, bool JustChecking)
 {
-    register int	fd;
-    register int	i;
-    PID_T		pid;
+    int	fd;
+    int	i;
+    pid_t		pid;
     char		buff[BUFSIZ];
 
     /* Open the file. */
     if ((fd = open(name, O_RDONLY)) < 0) {
 	if (JustChecking)
-	    return FALSE;
-	(void)fprintf(stderr, CANTOPEN, name, strerror(errno));
-	return TRUE;
+	    return false;
+        syswarn("cannot open %s", name);
+	return true;
     }
 
     /* Read the PID that is written there. */
     if (BinaryLock) {
 	if (read(fd, (char *)&pid, sizeof pid) != sizeof pid) {
-	    (void)close(fd);
-	    return FALSE;
+	    close(fd);
+	    return false;
 	}
     }
     else {
 	if ((i = read(fd, buff, sizeof buff - 1)) <= 0) {
-	    (void)close(fd);
-	    return FALSE;
+	    close(fd);
+	    return false;
 	}
 	buff[i] = '\0';
-	pid = (PID_T) atol(buff);
+	pid = (pid_t) atol(buff);
     }
-    (void)close(fd);
+    close(fd);
     if (pid <= 0)
-	return FALSE;
+	return false;
 
     /* Send the signal. */
     if (kill(pid, 0) < 0 && errno == ESRCH)
-	return FALSE;
+	return false;
 
     /* Either the kill worked, or we're optimistic about the error code. */
-    return TRUE;
+    return true;
 }
 
 
 /*
 **  Unlink a file, print a message on error, and exit.
 */
-STATIC NORETURN
-UnlinkAndExit(name, x)
-    char	*name;
-    int		x;
+static void
+UnlinkAndExit(char *name, int x)
 {
     if (unlink(name) < 0)
-	(void)fprintf(stderr, CANTUNLINK, name, strerror(errno));
+        syswarn("cannot unlink %s", name);
     exit(x);
 }
 
@@ -84,35 +80,35 @@ UnlinkAndExit(name, x)
 /*
 **  Print a usage message and exit.
 */
-STATIC NORETURN
-Usage()
+static void
+Usage(void)
 {
-    (void)fprintf(stderr, "Usage: shlock [-u|-b] -f file -p pid\n");
+    fprintf(stderr, "Usage: shlock [-u|-b] -f file -p pid\n");
     exit(1);
-    /* NOTREACHED */
 }
 
 
 int
-main(ac, av)
-    int			ac;
-    char		*av[];
+main(int ac, char *av[])
 {
-    register int	i;
-    register char	*p;
-    register int	fd;
+    int	i;
+    char	*p;
+    int	fd;
     char		tmp[BUFSIZ];
     char		buff[BUFSIZ];
     char		*name;
-    PID_T		pid;
-    BOOL		ok;
-    BOOL		JustChecking;
+    pid_t		pid;
+    bool		ok;
+    bool		JustChecking;
+
+    /* Establish our identity. */
+    message_program_name = "shlock";
 
     /* Set defaults. */
     pid = 0;
     name = NULL;
-    JustChecking = FALSE;
-    (void)umask(NEWSUMASK);
+    JustChecking = false;
+    umask(NEWSUMASK);
 
     /* Parse JCL. */
     while ((i = getopt(ac, av, "bcup:f:")) != EOF)
@@ -122,13 +118,13 @@ main(ac, av)
 	    /* NOTREACHED */
 	case 'b':
 	case 'u':
-	    BinaryLock = TRUE;
+	    BinaryLock = true;
 	    break;
 	case 'c':
-	    JustChecking = TRUE;
+	    JustChecking = true;
 	    break;
 	case 'p':
-	    pid = (PID_T) atol(optarg);
+	    pid = (pid_t) atol(optarg);
 	    break;
 	case 'f':
 	    name = optarg;
@@ -142,48 +138,44 @@ main(ac, av)
     /* Create the temp file in the same directory as the destination. */
     if ((p = strrchr(name, '/')) != NULL) {
 	*p = '\0';
-	(void)sprintf(tmp, "%s/shlock%ld", name, (long)getpid());
+	snprintf(tmp, sizeof(tmp), "%s/shlock%ld", name, (long)getpid());
 	*p = '/';
     }
     else
-	(void)sprintf(tmp, "shlock%ld", (long)getpid());
+	snprintf(tmp, sizeof(tmp), "shlock%ld", (long)getpid());
 
     /* Loop until we can open the file. */
     while ((fd = open(tmp, O_RDWR | O_CREAT | O_EXCL, 0644)) < 0)
 	switch (errno) {
 	default:
 	    /* Unknown error -- give up. */
-	    (void)fprintf(stderr, CANTOPEN, tmp, strerror(errno));
-	    exit(1);
+            sysdie("cannot open %s", tmp);
 	case EEXIST:
 	    /* If we can remove the old temporary, retry the open. */
-	    if (unlink(tmp) < 0) {
-		(void)fprintf(stderr, CANTUNLINK, tmp, strerror(errno));
-		exit(1);
-	    }
+	    if (unlink(tmp) < 0)
+                sysdie("cannot unlink %s", tmp);
 	    break;
 	}
 
     /* Write the process ID. */
     if (BinaryLock)
-	ok = write(fd, (POINTER)&pid, (SIZE_T)sizeof pid) == sizeof pid;
+	ok = write(fd, &pid, sizeof pid) == sizeof pid;
     else {
-	(void)sprintf(buff, "%ld\n", (long) pid);
+	snprintf(buff, sizeof(buff), "%ld\n", (long) pid);
 	i = strlen(buff);
-	ok = write(fd, (POINTER)buff, (SIZE_T)i) == i;
+	ok = write(fd, buff, i) == i;
     }
     if (!ok) {
-	(void)fprintf(stderr, "Can't write PID to \"%s\", %s\n",
-	    tmp, strerror(errno));
-	(void)close(fd);
+        syswarn("cannot write PID to %s", tmp);
+	close(fd);
 	UnlinkAndExit(tmp, 1);
     }
 
-    (void)close(fd);
+    close(fd);
 
     /* Handle the "-c" flag. */
     if (JustChecking) {
-	if (ValidLock(name, TRUE))
+	if (ValidLock(name, true))
 	    UnlinkAndExit(tmp, 1);
 	UnlinkAndExit(tmp, 0);
     }
@@ -193,20 +185,20 @@ main(ac, av)
 	switch (errno) {
 	default:
 	    /* Unknown error -- give up. */
-	    (void)fprintf(stderr, "Can't link \"%s\" to \"%s\", %s\n",
-		    tmp, name, strerror(errno));
+            syswarn("cannot link %s to %s", tmp, name);
 	    UnlinkAndExit(tmp, 1);
 	    /* NOTREACHED */
 	case EEXIST:
 	    /* File exists; if lock is valid, give up. */
-	    if (ValidLock(name, FALSE))
+	    if (ValidLock(name, false))
 		UnlinkAndExit(tmp, 1);
 	    if (unlink(name) < 0) {
-		(void)fprintf(stderr, CANTUNLINK, name, strerror(errno));
+                syswarn("cannot unlink %s", name);
 		UnlinkAndExit(tmp, 1);
 	    }
 	}
 
     UnlinkAndExit(tmp, 0);
     /* NOTREACHED */
+    return 1;
 }
