@@ -1,45 +1,18 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Wed Nov 29 23:44:47 1995
- * Project:     INN (innfeed)
- * File:        misc.h
- * RCSId:       $Id: misc.h,v 1.4 1998/03/23 16:10:06 mibsoft Exp $
- *
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Description: 
- * 
- */
+/*  $Id: misc.h 6648 2004-01-25 20:07:11Z rra $
+**
+**  Miscellaneous utility functions for innfeed.
+**
+**  Written by James Brister <brister@vix.com>
+*/
 
 #if ! defined ( misc_h__ )
 #define misc_h__
 
 #include "config.h"
 
+#include <stdarg.h>
 #include <sys/types.h>
 
-#if defined (DO_NEED_BOOL)
-typedef enum { false = 0, true = 1 } bool ;
-#endif
-
-
 /* These typedefs are all here because C is too stupid to let me multiply
    define typedefs to the same things (as C++ will). Hence I can't redeclare
    the typedefs to get around recursive header file includes (like host.h and
@@ -66,24 +39,14 @@ typedef void (*EndpTCB) (TimeoutId tid, void *d) ; /* see endpoint.h */
 typedef void (*EndpWorkCbk) (EndPoint ep, void *data) ;
 
 
-#if defined (DO_NEED_U_INT)
-typedef unsigned long u_long ;
-typedef unsigned int u_int ;
-typedef unsigned short u_short ;
-#endif
-
 /* debugging information */
-extern char *program ;
-extern u_int loggingLevel ;     /* if 0 then d_printf is a no-op */
+extern unsigned int loggingLevel ;     /* if 0 then d_printf is a no-op */
 
 /* the current count of file desccriptors */
-extern u_int openfds ;
-
-/* return the maximum number of fds this process can have. */
-int maxFds (void) ;
+extern unsigned int openfds ;
 
 /* if level <= loggingLevel then print */
-void d_printf (u_int level, const char *fmt, ...) __attribute__ ((__format__ (printf, 2, 3)));
+void d_printf (unsigned int level, const char *fmt, ...) __attribute__ ((__format__ (printf, 2, 3)));
 
 /* for the gethostbyname() error code */
 const char *host_err_str (void) ;
@@ -104,12 +67,13 @@ char *findNonBlankString (char *ptr, char **tail) ;
 void logOrPrint (int level, FILE *fp, const char *fmt, ...)
   __attribute__ ((__format__ (printf, 3, 4)));
 
-/* print the arguments like printf to stderr then abort() */
-void die (const char *fmt, ...) __attribute__ ((__format__ (printf, 1, 2)));
+/* Error handling functions for use with warn and die. */
+void error_log_stderr_date(int len, const char *fmt, va_list args, int err);
 
-/* print the arguments like printf to stderr. */
-void warn (const char *fmt, ...)  __attribute__ ((__format__ (printf, 1, 2)));
+/* Do cleanup and then abort, for use with die. */
+int dump_core(void);
 
+/* Alternate die that doesn't invoke an error handler. */
 void logAndExit (int exitVal, const char *fmt, ...)
   __attribute__ ((__format__ (printf, 2, 3)));
 
@@ -124,10 +88,6 @@ char *mystrtok (char *string, const char *sep) ;
 /* remove trailing whitespace */
 void trim_ws (char *string) ;
 
-#if defined (DO_NEED_STRDUP)
-char *strdup (const char *s) ;
-#endif
-
 /* locks the peer and returns true or returns false */
 bool lockPeer (const char *peerName) ;
 
