@@ -1,66 +1,31 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Sun Dec 31 18:52:04 1995
- * Project:     INN (innfeed)
- * File:        tape.c
- * RCSId:       $Id: tape.c,v 1.10 1998/06/19 11:58:18 scrappy Exp $
- *
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Description: The implementation of the Tape class. Tapes are read-only
- *              or write-only files that are accessed sequentially. Their
- *              basic unit of i/o is an Article. Tapes work out of a single
- *              directory and manage all file names themselves.
- *
- *              Tapes will checkpoint themselves periodically so
- *              that when innfeed exits or crashes things can
- *              restart close to where they were last. The period
- *              checkpointing is handled entirely by the Tape class,
- *              but the checkpoint period needs to be set by some
- *              external user before the first tape is created.
- *
- */
-
- 
-#if ! defined (lint)
-static const char *rcsid = "$Id: tape.c,v 1.10 1998/06/19 11:58:18 scrappy Exp $" ;
-static void use_rcsid (const char *rid) {   /* Never called */
-  use_rcsid (rcsid) ; use_rcsid (rid) ;
-}
-#endif
+/*  $Id: tape.c 6716 2004-05-16 20:26:56Z rra $
+**
+**  The implementation of the innfeed Tape class.
+**
+**  Written by James Brister <brister@vix.com>
+**
+**  The implementation of the Tape class. Tapes are read-only or write-only
+**  files that are accessed sequentially. Their basic unit of i/o is an
+**  Article. Tapes work out of a single directory and manage all file names
+**  themselves.
+**
+**  Tapes will checkpoint themselves periodically so that when innfeed exits
+**  or crashes things can restart close to where they were last. The period
+**  checkpointing is handled entirely by the Tape class, but the checkpoint
+**  period needs to be set by some external user before the first tape is
+**  created.
+*/
 
+#include "innfeed.h"
 #include "config.h"
+#include "clibrary.h"
 
-#if defined (HAVE_UNISTD_H)
-#include <unistd.h>
-#endif
-
-#include <stdlib.h>
+#include <assert.h>
+#include <ctype.h>
+#include <errno.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <stdio.h>
-#include <assert.h>
-#include <string.h>
-#include <ctype.h>
 #include <syslog.h>
-#include <errno.h>
 
 #if defined (HAVE_DIRENT_H)
 #include <dirent.h>
@@ -70,21 +35,28 @@ typedef struct dirent DIRENTRY ;
 typedef struct direct DIRENTRY ;
 #endif
 
-#if defined (DO_NEED_TIME)
-#include <time.h>
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
 #endif
-#include <sys/time.h>
 
-#include "tape.h"
-#include "article.h"
-#include "endpoint.h"
-#include "msgs.h"
-#include "configfile.h"
-#include "configdata.h"
-#include "clibrary.h"
+#include "inn/innconf.h"
+#include "inn/messages.h"
 #include "libinn.h"
 
+#include "article.h"
+#include "configfile.h"
+#include "endpoint.h"
+#include "tape.h"
+
 extern char *dflTapeDir;
+extern bool genHtml ;
 
 #if 0
 /* a structure for temporary storage of articles. */
@@ -124,7 +96,7 @@ struct tape_s
     /* the tape holds a small output queue in memory to avoid thrashing. */
     QueueElem head ;            
     QueueElem tail ;
-    u_int qLength ;             /* amount on the queue */
+    unsigned int qLength ;             /* amount on the queue */
 #endif
     
     long outputSize ;           /* the current size of the output file. */
@@ -174,11 +146,8 @@ static TimeoutId checkPtId ;
 static TimeoutId ckNewFileId ;
 
 /* number of seconds between tape checkpoints. */
-static u_int tapeCkPtPeriod ;
-static u_int tapeCkNewFilePeriod ;
-
-/* the callback ID of the tape rotation timer callback. */
-static TimeoutId rotateId ;     /* XXX */
+static unsigned int tapeCkPtPeriod ;
+static unsigned int tapeCkNewFilePeriod ;
 
 static time_t rotatePeriod = TAPE_ROTATE_PERIOD ;
 
@@ -196,7 +165,7 @@ static size_t activeTapeIdx ;
 static long defaultSizeLimit ;
 #endif
 
-u_int tapeHighwater ;
+unsigned int tapeHighwater ;
 
 bool debugShrinking = false ;
 
@@ -220,27 +189,34 @@ int tapeConfigLoadCbk (void *data)
       free(p);
       if (tapeDirectory != NULL && strcmp (tapeDirectory,dir) != 0)
         {
-          syslog (LOG_ERR,NO_CHANGE_BACKLOG) ;
-          FREE (dir) ;
-          dir = strdup (tapeDirectory) ;
+          warn ("ME config: cannot change backlog-directory of a running"
+                " process") ;
+          free (dir) ;
+          dir = xstrdup (tapeDirectory) ;
         }
 
       if (!isDirectory (dir) && isDirectory (dflTapeDir))
         {
-          logOrPrint (LOG_ERR,fp,BAD_TAPEDIR_CHANGE,dir,dflTapeDir) ;
-          FREE (dir) ;
-          dir = strdup (dflTapeDir) ;
+          logOrPrint (LOG_ERR,fp,
+                      "ME config: definition of backlog-directory (%s) is a"
+                      " non-existant directory. Using %s",
+                      dir,dflTapeDir) ;
+          free (dir) ;
+          dir = xstrdup (dflTapeDir) ;
         }
       else if (!isDirectory (dir))
-        logAndExit (1,NO_TAPE_DIR) ;
+        logAndExit (1,"ME config: no usable value for backlog-directory") ;
     }
   else if (!isDirectory (dflTapeDir))
-    logAndExit (1,NO_TAPE_DIR) ;
+  {
+    logAndExit (1,"ME config: no usable value for backlog-directory") ;
+    return -1; /* not reached */
+  }
   else
-    dir = strdup (dflTapeDir) ;
+    dir = xstrdup (dflTapeDir) ;
   
   if (tapeDirectory != NULL)
-    FREE (tapeDirectory) ;
+    free (tapeDirectory) ;
   tapeDirectory = dir ;
 
 
@@ -250,14 +226,16 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-highwater",
+          logOrPrint (LOG_ERR,fp,
+                      "ME config: value of %s (%ld) in %s cannot be less"
+                      " than 0. Using %ld","backlog-highwater",
                       iv,"global scope",(long)TAPE_HIGHWATER);
           iv = TAPE_HIGHWATER ;
         }
     }
   else
     iv = TAPE_HIGHWATER ;
-  tapeHighwater = (u_int) iv ;
+  tapeHighwater = (unsigned int) iv ;
 
 
   
@@ -266,14 +244,17 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-rotate-period",
+          logOrPrint (LOG_ERR,fp,
+                      "ME config: value of %s (%ld) in %s cannot be less"
+                      " than 0. Using %ld",
+                      "backlog-rotate-period",
                       iv,"global scope",(long)TAPE_ROTATE_PERIOD);
           iv = TAPE_ROTATE_PERIOD ;
         }
     }
   else
     iv = TAPE_ROTATE_PERIOD ;
-  rotatePeriod = (u_int) iv ;
+  rotatePeriod = (unsigned int) iv ;
 
 
   if (getInteger (topScope,"backlog-ckpt-period",&iv,NO_INHERIT))
@@ -281,14 +262,17 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-ckpt-period",iv,
+          logOrPrint (LOG_ERR,fp,
+                      "ME config: value of %s (%ld) in %s cannot be less"
+                      " than 0. Using %ld",
+                      "backlog-ckpt-period",iv,
                       "global scope",(long)TAPE_CHECKPOINT_PERIOD);
           iv = TAPE_CHECKPOINT_PERIOD ;
         }
     }
   else
     iv = TAPE_CHECKPOINT_PERIOD ;
-  tapeCkPtPeriod = (u_int) iv ;
+  tapeCkPtPeriod = (unsigned int) iv ;
 
 
   if (getInteger (topScope,"backlog-newfile-period",&iv,NO_INHERIT))
@@ -296,14 +280,17 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-newfile-period",
+          logOrPrint (LOG_ERR,fp,
+                      "ME config: value of %s (%ld) in %s cannot be less"
+                      " than 0. Using %ld",
+                      "backlog-newfile-period",
                       iv,"global scope",(long)TAPE_NEWFILE_PERIOD);
           iv = TAPE_NEWFILE_PERIOD ;
         }
     }
   else
     iv = TAPE_NEWFILE_PERIOD ;
-  tapeCkNewFilePeriod = (u_int) iv ;
+  tapeCkNewFilePeriod = (unsigned int) iv ;
 
 
   if (getBool (topScope,"debug-shrinking",&bv,NO_INHERIT))
@@ -324,7 +311,7 @@ int tapeConfigLoadCbk (void *data)
 static bool inited = false ;
 Tape newTape (const char *peerName, bool dontRotate)
 {
-  Tape nt = ALLOC (struct tape_s, 1) ;
+  Tape nt = xmalloc (sizeof(struct tape_s)) ;
   size_t pLen = strlen (peerName) ;
   size_t dLen = strlen (tapeDirectory) ;
 
@@ -345,33 +332,29 @@ Tape newTape (const char *peerName, bool dontRotate)
   if (endsIn (peerName,LOCK_TAIL))
     die ("Sorry, can't have a peer name ending in \"%s\"",LOCK_TAIL) ;
 
-  nt->peerName = strdup (peerName) ;
+  nt->peerName = xstrdup (peerName) ;
   
-  nt->handFilename = MALLOC (pLen + dLen + 2) ;
-  ASSERT (nt->handFilename != NULL) ;
+  nt->handFilename = xmalloc (pLen + dLen + 2) ;
   sprintf (nt->handFilename,"%s/%s",tapeDirectory,peerName) ;
 
-  nt->lockFilename = MALLOC (pLen + dLen + strlen(LOCK_TAIL) + 2) ;
-  ASSERT (nt->lockFilename != NULL) ;
+  nt->lockFilename = xmalloc (pLen + dLen + strlen(LOCK_TAIL) + 2) ;
   sprintf (nt->lockFilename,"%s/%s%s",tapeDirectory,peerName,LOCK_TAIL) ;
 
-  nt->inputFilename = MALLOC (pLen + dLen + strlen(INPUT_TAIL) + 2) ;
-  ASSERT (nt->inputFilename != NULL) ;
+  nt->inputFilename = xmalloc (pLen + dLen + strlen(INPUT_TAIL) + 2) ;
   sprintf (nt->inputFilename,"%s/%s%s",tapeDirectory,peerName,INPUT_TAIL) ;
 
-  nt->outputFilename = MALLOC (pLen + dLen + strlen(OUTPUT_TAIL) + 2) ;
-  ASSERT (nt->outputFilename != NULL) ;
+  nt->outputFilename = xmalloc (pLen + dLen + strlen(OUTPUT_TAIL) + 2) ;
   sprintf (nt->outputFilename,"%s/%s%s",tapeDirectory,peerName,OUTPUT_TAIL) ;
 
   if ( !lockFile (nt->lockFilename) )
     {
-      syslog (LOG_ERR,NO_LOCK_TAPE,nt->lockFilename) ;
+      warn ("ME lock failed for host: %s", nt->lockFilename) ;
       
-      FREE (nt->handFilename) ;
-      FREE (nt->lockFilename) ;
-      FREE (nt->inputFilename) ;
-      FREE (nt->outputFilename) ;
-      FREE (nt) ;
+      free (nt->handFilename) ;
+      free (nt->lockFilename) ;
+      free (nt->inputFilename) ;
+      free (nt->outputFilename) ;
+      free (nt) ;
 
       return NULL ;
     }
@@ -437,7 +420,8 @@ static void initTape (Tape nt)
               if (!getInteger (s,"backlog-limit-highwater",
                                &nt->outputHighLimit,INHERIT))
                 {
-                  syslog (LOG_WARNING,NO_FACTOR,nt->peerName) ;
+                  warn ("%s no backlog-factor or backlog-high-limit",
+                        nt->peerName) ;
                   nt->outputLowLimit = 0 ;
                   nt->outputHighLimit = 0 ;
                   nt->backlogFactor = 0.0 ;
@@ -447,7 +431,7 @@ static void initTape (Tape nt)
             nt->outputHighLimit = (long)(nt->outputLowLimit * nt->backlogFactor);
         }
       else
-        syslog (LOG_ERR,NODEFN,"backlog-limit") ;
+        warn ("ME config: no definition for required key backlog-limit") ;
     }
   
   d_printf (1, "%s spooling: %s\n", nt->peerName, 
@@ -463,9 +447,9 @@ static void initTape (Tape nt)
 
 void gFlushTapes (void)
 {
-  u_int i ;
+  unsigned int i ;
 
-  syslog (LOG_NOTICE,FLUSHING_TAPES) ;
+  notice ("ME flushing tapes") ;
   for (i = 0 ; i < activeTapeIdx ; i++)
     tapeFlush (activeTapes [i]) ;
 }
@@ -490,17 +474,17 @@ void tapeFlush (Tape tape)
 
 
 
-void gPrintTapeInfo (FILE *fp, u_int indentAmt)
+void gPrintTapeInfo (FILE *fp, unsigned int indentAmt)
 {
   char indent [INDENT_BUFFER_SIZE] ;
-  u_int i ;
+  unsigned int i ;
   
   for (i = 0 ; i < MIN(INDENT_BUFFER_SIZE - 1,indentAmt) ; i++)
     indent [i] = ' ' ;
   indent [i] = '\0' ;
 
-  fprintf (fp,"%sGlobal Tape List : (count %d) {\n",
-           indent,activeTapeIdx) ;
+  fprintf (fp,"%sGlobal Tape List : (count %lu) {\n",
+           indent,(unsigned long) activeTapeIdx) ;
 
   for (i = 0 ; i < activeTapeIdx ; i++)
     printTapeInfo (activeTapes [i],fp,indentAmt + INDENT_INCR) ;
@@ -510,7 +494,8 @@ void gPrintTapeInfo (FILE *fp, u_int indentAmt)
 
 void tapeLogGlobalStatus (FILE *fp)
 {
-  fprintf (fp,"Backlog file global values\n") ;
+  fprintf (fp,"%sBacklog file global values:%s\n",
+           genHtml ? "<B>" : "", genHtml ? "</B>" : "") ;
   fprintf (fp,"        directory: %s\n",tapeDirectory) ;
   fprintf (fp,"    rotate period: %-3ld seconds\n",(long) rotatePeriod) ;
   fprintf (fp,"checkpoint period: %-3ld seconds\n",(long) tapeCkPtPeriod) ;
@@ -525,6 +510,8 @@ void tapeLogStatus (Tape tape, FILE *fp)
     fprintf (fp,"(no tape)\n") ;
   else if (tape->noBacklog)
       fprintf (fp,"                 spooling: DISABLED\n");
+  else if (tape->outputLowLimit == 0)
+      fprintf (fp,"                 spooling: UNLIMITED\n");
   else 
     {
       fprintf (fp,"                 backlog low limit: %ld\n",
@@ -539,10 +526,10 @@ void tapeLogStatus (Tape tape, FILE *fp)
     }
 }
 
-void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
+void printTapeInfo (Tape tape, FILE *fp, unsigned int indentAmt)
 {
   char indent [INDENT_BUFFER_SIZE] ;
-  u_int i ;
+  unsigned int i ;
 #if 0
   QueueElem qe ;
 #endif
@@ -551,7 +538,7 @@ void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
     indent [i] = ' ' ;
   indent [i] = '\0' ;
 
-  fprintf (fp,"%sTape : %p {\n",indent,tape) ;
+  fprintf (fp,"%sTape : %p {\n",indent,(void *) tape) ;
 
   if (tape == NULL)
     {
@@ -564,8 +551,8 @@ void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
   fprintf (fp,"%s    output-file : %s\n",indent, tape->outputFilename) ;
   fprintf (fp,"%s    lock-file : %s\n",indent, tape->lockFilename) ;
   fprintf (fp,"%s    peerName : %s\n",indent,tape->peerName) ;
-  fprintf (fp,"%s    input-FILE : %p\n",indent, tape->inFp) ;
-  fprintf (fp,"%s    output-FILE : %p\n",indent, tape->outFp) ;
+  fprintf (fp,"%s    input-FILE : %p\n",indent, (void *) tape->inFp) ;
+  fprintf (fp,"%s    output-FILE : %p\n",indent, (void *) tape->outFp) ;
   fprintf (fp,"%s    output-limit : %ld\n",indent, tape->outputLowLimit) ;
 
 #if 0
@@ -607,12 +594,12 @@ void delTape (Tape tape)
 #if 1
   
   if (tape->outFp != NULL && fclose (tape->outFp) != 0)
-    syslog (LOG_ERR,FCLOSE_FAILED, tape->outputFilename) ;
+    syswarn ("ME ioerr fclose %s", tape->outputFilename) ;
 
   if (stat(tape->outputFilename, &st) == 0 && st.st_size == 0)
     {
       d_printf (1,"removing empty output tape: %s\n",tape->outputFilename) ;
-      (void) unlink (tape->outputFilename) ;
+      unlink (tape->outputFilename) ;
     }
   
   tape->outFp = NULL ;
@@ -627,7 +614,7 @@ void delTape (Tape tape)
   if (tape->inFp != NULL)
     {
       checkpointTape (tape) ;
-      (void) fclose (tape->inFp) ;
+      fclose (tape->inFp) ;
     }
 
   unlockFile (tape->lockFilename) ;
@@ -640,7 +627,7 @@ void delTape (Tape tape)
              
   removeTapeGlobally (tape) ;
   
-  FREE (tape) ;
+  free (tape) ;
 }
 
 
@@ -674,7 +661,7 @@ void tapeTakeArticle (Tape tape, Article article)
 #else
 #if defined (NO_TRUST_STRLEN)
 
-  tape->outputSize = ftell (tape->outFp) ;
+  tape->outputSize = ftello (tape->outFp) ;
 
 #else
 
@@ -692,18 +679,17 @@ void tapeTakeArticle (Tape tape, Article article)
       fflush (tape->outFp) ;
       
       if (fstat (fileno (tape->outFp),&sb) != 0)
-        syslog (LOG_ERR,FSTAT_FAILURE,tape->outputFilename) ;
+        syswarn ("ME oserr fstat %s",tape->outputFilename) ;
       else if (sb.st_size != tape->outputSize) 
-        syslog (LOG_ERR,"fstat and ftell do not agree: %ld %ld for %s\n",
+        syslog (LOG_ERR,"fstat and ftello do not agree: %ld %ld for %s\n",
                 (long)sb.st_size,tape->outputSize,tape->outputFilename) ;
     }
   
   if (tape->outputHighLimit > 0 && tape->outputSize >= tape->outputHighLimit)
     {
       long oldSize = tape->outputSize ;
-      (void) shrinkfile (tape->outFp,
-                         tape->outputLowLimit,tape->outputFilename,"a+") ;
-      tape->outputSize = ftell (tape->outFp) ;
+      shrinkfile (tape->outFp,tape->outputLowLimit,tape->outputFilename,"a+");
+      tape->outputSize = ftello (tape->outFp) ;
       tape->lossage += oldSize - tape->outputSize ;
     }
 }
@@ -731,19 +717,19 @@ Article getArticle (Tape tape)
       if (fgets (line,sizeof (line), tape->inFp) == NULL)
         {
           if (ferror (tape->inFp))
-            syslog (LOG_ERR,TAPE_INPUT_ERROR,tape->inputFilename) ;
+            syswarn ("ME ioerr on tape file %s", tape->inputFilename) ;
           else if ( !feof (tape->inFp) )
-            syslog (LOG_ERR,FGETS_FAILED,tape->inputFilename) ;
+            syswarn ("ME oserr fgets %s", tape->inputFilename) ;
           
           if (fclose (tape->inFp) != 0)
-            syslog (LOG_ERR,FCLOSE_FAILED, tape->inputFilename);
+            syswarn ("ME ioerr fclose %s", tape->inputFilename) ;
 
           d_printf (1,"No more articles on tape %s\n",tape->inputFilename) ;
 
           tape->inFp = NULL ;
           tape->scribbled = false ;
 
-          (void) unlink (tape->inputFilename) ;
+          unlink (tape->inputFilename) ;
 
           if ((now - tape->lastRotated) > rotatePeriod)
             prepareFiles (tape) ; /* rotate files to try next. */
@@ -752,7 +738,7 @@ Article getArticle (Tape tape)
         {
           msgid = filename = NULL ;
           
-          for (p = line ; *p && isspace (*p) ; p++) /* eat whitespace */
+          for (p = line ; *p && CTYPE(isspace, *p) ; p++) /* eat whitespace */
             /* nada */ ;
 
           if (*p != '\0')
@@ -764,7 +750,7 @@ Article getArticle (Tape tape)
                   filename = p ;
                   *q = '\0' ;
       
-                  for (q++ ; *q && isspace (*q) ; q++) /* eat more white */
+                  for (q++ ; *q && CTYPE(isspace, *q) ; q++)
                     /* nada */ ;
 
                   if (*q != '\0')
@@ -791,7 +777,8 @@ Article getArticle (Tape tape)
               ;
             if (p > msgid) p--;
             if (*msgid != '<' || *p != '>') {
-              syslog (LOG_ERR,TAPE_SPAGHETTI,tape->inputFilename,msgid);
+              warn ("ME tape invalid messageID in %s: %s",
+                    tape->inputFilename,msgid);
               msgid = NULL;
             }
           }
@@ -813,13 +800,13 @@ Article getArticle (Tape tape)
         {
           /* last article read was the end of the tape. */
           if (fclose (tape->inFp) != 0)
-            syslog (LOG_ERR,FCLOSE_FAILED,tape->inputFilename) ;
+            syswarn ("ME ioerr fclose %s", tape->inputFilename) ;
           
           tape->inFp = NULL ;
           tape->scribbled = false ;
           
           /* toss out the old input file and prepare the new one */
-          (void) unlink (tape->inputFilename) ;
+          unlink (tape->inputFilename) ;
           
           if (now - tape->lastRotated > rotatePeriod)
             prepareFiles (tape) ;
@@ -844,7 +831,7 @@ Article getArticle (Tape tape)
 /* Cause all the Tapes to checkpoint themselves. */
 void checkPointTapes (void)
 {
-  u_int i ;
+  unsigned int i ;
 
   for (i = 0 ; i < activeTapeIdx ; i++)
     checkpointTape (activeTapes [i]) ;
@@ -854,7 +841,7 @@ void checkPointTapes (void)
 /* make all the tapes set their checkNew flag. */
 static void tapesSetCheckNew (void) 
 {
-  u_int i ;
+  unsigned int i ;
 
   for (i = 0 ; i < activeTapeIdx ; i++)
     activeTapes[i]->checkNew = true ;
@@ -876,7 +863,7 @@ void setTapeDirectory (const char *newDir)
   if (tapeDirectory != NULL)
     freeCharP (tapeDirectory) ;
   
-  tapeDirectory = strdup (newDir) ;
+  tapeDirectory = xstrdup (newDir) ;
 
   addPointerFreedOnExit (tapeDirectory) ;
 }
@@ -890,7 +877,7 @@ const char *getTapeDirectory (void)
 #if 0
   if (tapeDirectory == NULL)
     {
-      tapeDirectory = strdup (dflTapeDir) ;
+      tapeDirectory = xstrdup (dflTapeDir) ;
       addPointerFreedOnExit (tapeDirectory) ;
     }
 #endif
@@ -923,15 +910,13 @@ static void addTapeGlobally (Tape tape)
   
   if (activeTapeSize == activeTapeIdx)
     {
-      u_int i ;
+      unsigned int i ;
       
       activeTapeSize += 10 ;
       if (activeTapes != NULL)
-        activeTapes = REALLOC (activeTapes, Tape, activeTapeSize) ;
+        activeTapes = xrealloc (activeTapes, sizeof(Tape) * activeTapeSize) ;
       else
-        activeTapes = ALLOC (Tape, activeTapeSize) ;
-
-      ASSERT (activeTapes != NULL) ;
+        activeTapes = xmalloc (sizeof(Tape) * activeTapeSize) ;
 
       for (i = activeTapeIdx ; i < activeTapeSize ; i++)
         activeTapes [i] = NULL ;
@@ -943,7 +928,7 @@ static void addTapeGlobally (Tape tape)
 /* Remove a tape for the class-level list of active tapes. */
 static void removeTapeGlobally (Tape tape)
 {
-  u_int i ;
+  unsigned int i ;
 
   if (tape == NULL)
     return ;
@@ -963,7 +948,7 @@ static void removeTapeGlobally (Tape tape)
 
   if (activeTapeIdx == 0)
     {
-      FREE (activeTapes) ;
+      free (activeTapes) ;
       activeTapes = NULL ;
     }
 }
@@ -982,10 +967,9 @@ static void checkpointTape (Tape tape)
       return ;
     }
   
-  if ((tape->tellpos = ftell (tape->inFp)) < 0)
+  if ((tape->tellpos = ftello (tape->inFp)) < 0)
     {
-      syslog (LOG_ERR,FTELL_FAILED,tape->inputFilename) ;
-
+      syswarn ("ME oserr ftello %s", tape->inputFilename) ;
       return ;
     }
 
@@ -1022,8 +1006,9 @@ static void checkpointTape (Tape tape)
       
       fprintf (tape->inFp,"%ld",tape->tellpos) ;
 
-      if (fseek (tape->inFp,tape->tellpos,SEEK_SET) != 0)
-        syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos) ;
+      if (fseeko (tape->inFp,tape->tellpos,SEEK_SET) != 0)
+        syswarn ("ME oserr fseeko(%s,%ld,SEEK_SET)",tape->inputFilename,
+                 tape->tellpos) ;
     }
   
   tape->changed = false ;
@@ -1073,7 +1058,7 @@ static void prepareFiles (Tape tape)
     {
       newExists = fileExistsP (tape->handFilename) ;
       if (newExists)
-        syslog (LOG_NOTICE,NEW_HAND_FILE,tape->peerName) ;
+        notice ("%s new hand-prepared backlog file", tape->peerName) ;
     }
   else
     newExists = false ;
@@ -1094,10 +1079,11 @@ static void prepareFiles (Tape tape)
   if (newExists && !inpExists)
     {
       if (rename (tape->handFilename,tape->inputFilename) != 0)
-        syslog (LOG_ERR,RENAME_FAILED,tape->handFilename, tape->inputFilename);
+        syswarn ("ME oserr rename %s, %s", tape->handFilename,
+                 tape->inputFilename);
       else
         {
-          syslog (LOG_NOTICE,ROTATING_HAND_DROPPED,tape->peerName) ;
+          notice ("%s grabbing external tape file", tape->peerName) ;
           inpExists = true ;
         }
     }
@@ -1113,8 +1099,8 @@ static void prepareFiles (Tape tape)
         }
       
       if (rename (tape->outputFilename,tape->inputFilename) != 0)
-        syslog (LOG_ERR,RENAME_FAILED,
-                tape->outputFilename,tape->inputFilename);
+        syswarn ("ME oserr rename %s, %s", tape->outputFilename,
+                 tape->inputFilename) ;
       else
         inpExists = true ;
 
@@ -1128,7 +1114,7 @@ static void prepareFiles (Tape tape)
       long flength ;
       
       if ((tape->inFp = fopen (tape->inputFilename,"r+")) == NULL)
-        syslog (LOG_ERR,FOPEN_FAILURE,tape->inputFilename) ;
+        syswarn ("ME fopen %s", tape->inputFilename) ;
       else
         {
           char buffer [64] ;
@@ -1137,11 +1123,11 @@ static void prepareFiles (Tape tape)
             {
               if (feof (tape->inFp))
                 {
-                  d_printf (1,"Empty input file: %s",tape->inputFilename) ;
+                  d_printf (1,"Empty input file: %s\n",tape->inputFilename) ;
                   unlink (tape->inputFilename) ;
                 }
               else
-                syslog (LOG_ERR,FGETS_FAILED,tape->inputFilename) ;
+                syswarn ("ME oserr fgets %s", tape->inputFilename) ;
               
               fclose (tape->inFp) ;
               tape->inFp = NULL ;
@@ -1149,7 +1135,7 @@ static void prepareFiles (Tape tape)
             }
           else
             {
-              u_int len = strlen (buffer) ;
+              unsigned int len = strlen (buffer) ;
               long newPos = 0 ;
 
               if (len > 0 && buffer [len - 1] == '\n')
@@ -1166,14 +1152,15 @@ static void prepareFiles (Tape tape)
 
               if ((flength = fileLength (fileno (tape->inFp))) < tape->tellpos)
                 {
-                  syslog (LOG_ERR,FILE_SHORT,tape->inputFilename,
-                          flength, tape->tellpos);
+                  warn ("ME tape short: %s %ld %ld", tape->inputFilename,
+                        flength, tape->tellpos) ;
                   tape->tellpos = 0 ;
                 }
               else if (tape->tellpos == 0)
                 rewind (tape->inFp) ;
-              else if (fseek (tape->inFp,tape->tellpos - 1,SEEK_SET) != 0)
-                syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos);
+              else if (fseeko (tape->inFp,tape->tellpos - 1,SEEK_SET) != 0)
+                syswarn ("ME oserr fseeko(%s,%ld,SEEK_SET)",
+                         tape->inputFilename,tape->tellpos) ;
               else if ((c = fgetc (tape->inFp)) != '\n')
                 {
                   while (c != EOF && c != '\n')
@@ -1193,8 +1180,9 @@ static void prepareFiles (Tape tape)
                       
                       tape->changed = true ;
                       checkpointTape (tape) ;
-                      
-                      syslog (LOG_ERR,CKPT_BNDRY,tape->inputFilename,
+
+                      warn ("ME internal checkpoint line boundary missed:"
+                            " %s %ld vs. %ld",tape->inputFilename,
                               tape->tellpos, oldPos) ;
                     }
                 }
@@ -1210,20 +1198,18 @@ static void prepareFiles (Tape tape)
     {
       if ((tape->outFp = fopen (tape->outputFilename,"a+")) == NULL)
         {
-          syslog (LOG_ERR,TAPE_OPEN_FAILED, "a+", tape->outputFilename) ;
+          syswarn ("ME tape open failed (a+) %s", tape->outputFilename) ;
           return ;
         }
-      fseek (tape->outFp,0,SEEK_END) ;
-      tape->outputSize = ftell (tape->outFp) ;
+      fseeko (tape->outFp,0,SEEK_END) ;
+      tape->outputSize = ftello (tape->outFp) ;
       tape->lossage = 0 ;
     }
 }
 
 
-static void tapeCkNewFileCbk (TimeoutId id, void *d)
+static void tapeCkNewFileCbk (TimeoutId id, void *d UNUSED)
 {
-  (void) d ;
-  
   ASSERT (id == ckNewFileId) ;
   ASSERT (tapeCkNewFilePeriod > 0) ;
 
@@ -1234,10 +1220,8 @@ static void tapeCkNewFileCbk (TimeoutId id, void *d)
 
 
 /* The timer callback function that will checkpoint all the active tapes. */
-static void tapeCheckpointCallback (TimeoutId id, void *d)
+static void tapeCheckpointCallback (TimeoutId id, void *d UNUSED)
 {
-  (void) d ;                    /* keep lint happy */
-
   ASSERT (id == checkPtId) ;
   ASSERT (tapeCkPtPeriod > 0) ;
   
@@ -1265,7 +1249,7 @@ static void flushTape (Tape tape)
       
       delArticle (elem->article) ;
       
-      FREE (elem) ;
+      free (elem) ;
       elem = tape->head ;
     }
   tape->tail = NULL ;
@@ -1274,7 +1258,7 @@ static void flushTape (Tape tape)
 
   /* I'd rather know where I am each time, and I don't trust all
      fprintf's to give me character counts. */
-  tape->outputSize = ftell (tape->outFp) ;
+  tape->outputSize = ftello (tape->outFp) ;
   if (debugShrinking)
     {
       struct stat buf ;
@@ -1288,16 +1272,16 @@ static void flushTape (Tape tape)
         }
       else if (buf.st_size != tape->outputSize)
         {
-          syslog (LOG_ERR,FSTAT_NE_FTELL,tape->outputFilename) ;
+          warn ("ME fstat and ftello do not agree for %s",
+                tape->outputFilename) ;
           logged = true ;
         }
     }
   
   if (tape->outputHighLimit > 0 && tape->outputSize > tape->outputHighLimit)
     {
-      (void) shrinkfile (tape->outFp,
-                         tape->outputLowLimit,tape->outputFilename,"a+") ;
-      tape->outputSize = ftell (tape->outFp) ;
+      shrinkfile (tape->outFp,tape->outputLowLimit,tape->outputFilename,"a+");
+      tape->outputSize = ftello (tape->outFp) ;
     }
 }
 #endif
@@ -1305,5 +1289,6 @@ static void flushTape (Tape tape)
 
 static void tapeCleanup (void)
 {
-  FREE (tapeDirectory) ;
+  free (tapeDirectory) ;
+  tapeDirectory = NULL ;
 }
