@@ -1,14 +1,13 @@
-/*  $Revision: 1.3 $
+/*  $Id: resource.c 6135 2003-01-19 01:15:40Z rra $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
-#include "macros.h"
+#include "libinn.h"
 
+#ifdef HAVE_GETRUSAGE
 
-#if	defined(RES_RUSAGE)
 #include <sys/time.h>
 #include <sys/resource.h>
 
@@ -27,10 +26,9 @@ int GetResourceUsage(double *usertime, double *systime)
     *systime = TIMEVALasDOUBLE(R.ru_stime);
     return 0;
 }
-#endif	/* defined(RES_RUSAGE) */
 
+#else /* HAVE_GETRUSAGE */
 
-#if	defined(RES_TIMES)
 #include <sys/param.h>
 #include <sys/times.h>
 
@@ -50,4 +48,5 @@ int GetResourceUsage(double *usertime, double *systime)
     *systime = CPUTIMEasDOUBLE(T.tms_stime, T.tms_cstime);
     return 0;
 }
-#endif	/* defined(RES_TIMES) */
+
+#endif /* !HAVE_GETRUSAGE */
