@@ -1,27 +1,26 @@
-/*  $Id: timehash.c,v 1.28 1998/10/13 13:52:24 coneill Exp $
+/*  $Id: timehash.c 7412 2005-10-09 03:44:35Z eagle $
 **
-**  timehash based storing method
+**  Timehash based storage method.
 */
 
-#include <stdio.h>
-#include <errno.h>
-#include <unistd.h>
-#include <time.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
-#include <dirent.h>
+#include "config.h"
+#include "clibrary.h"
+#include "portable/mmap.h"
 #include <ctype.h>
-#include <configdata.h>
-#include <clibrary.h>
+#include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <netinet/in.h>
 #include <syslog.h>
-#include <macros.h>
-#include <libinn.h>
-#include <methods.h>
-#include "timehash.h"
+#include <sys/stat.h>
+#include <time.h>
+
+#include "inn/innconf.h"
+#include "inn/wire.h"
+#include "libinn.h"
+#include "methods.h"
 #include "paths.h"
+#include "timehash.h"
 
 typedef struct {
     char                *base;    /* Base of the mmaped file */
@@ -39,7 +38,7 @@ typedef enum {FIND_DIR, FIND_ART, FIND_TOPDIR} FINDTYPE;
 
 static int SeqNum = 0;
 
-static TOKEN MakeToken(time_t time, int seqnum, STORAGECLASS class, TOKEN *oldtoken) {
+static TOKEN MakeToken(time_t now, int seqnum, STORAGECLASS class, TOKEN *oldtoken) {
     TOKEN               token;
     unsigned int        i;
     unsigned short      s;
@@ -50,7 +49,7 @@ static TOKEN MakeToken(time_t time, int seqnum, STORAGECLASS class, TOKEN *oldto
 	memcpy(&token, oldtoken, sizeof(token));
     token.type = TOKEN_TIMEHASH;
     token.class = class;
-    i = htonl(time);
+    i = htonl(now);
     memcpy(token.token, &i, sizeof(i));
     if (sizeof(i) > 4)
 	memmove(token.token, &token.token[sizeof(i) - 4], 4);
@@ -59,49 +58,58 @@ static TOKEN MakeToken(time_t time, int seqnum, STORAGECLASS class, TOKEN *oldto
     return token;
 }
 
-static void BreakToken(TOKEN token, int *time, int *seqnum) {
+static void BreakToken(TOKEN token, int *now, int *seqnum) {
     unsigned int        i;
     unsigned short      s = 0;
 
     memcpy(&i, token.token, sizeof(i));
     memcpy(&s, &token.token[4], sizeof(s));
-    *time = ntohl(i);
+    *now = ntohl(i);
     *seqnum = (int)ntohs(s);
 }
 
-static char *MakePath(int time, int seqnum, const STORAGECLASS class) {
+static char *MakePath(int now, int seqnum, const STORAGECLASS class) {
     char *path;
+    size_t length;
     
     /* innconf->patharticles + '/time-zz/xx/xx/yyyy-xxxx' */
-    path = NEW(char, strlen(innconf->patharticles) + 32);
-    sprintf(path, "%s/time-%02x/%02x/%02x/%04x-%04x", innconf->patharticles,
-	    class, (time >> 16) & 0xff, (time >> 8) & 0xff, seqnum,
-	    (time & 0xff) | ((time >> 16 & 0xff00)));
+    length = strlen(innconf->patharticles) + 32;
+    path = xmalloc(length);
+    snprintf(path, length, "%s/time-%02x/%02x/%02x/%04x-%04x",
+             innconf->patharticles, class,
+             (now >> 16) & 0xff, (now >> 8) & 0xff, seqnum,
+             (now & 0xff) | ((now >> 16 & 0xff00)));
     return path;
 }
 
 static TOKEN *PathToToken(char *path) {
     int			n;
     unsigned int	t1, t2, t3, seqnum, class;
-    time_t		time;
+    time_t		now;
     static TOKEN	token;
 
     n = sscanf(path, "time-%02x/%02x/%02x/%04x-%04x", &class, &t1, &t2, &seqnum, &t3);
     if (n != 5)
 	return (TOKEN *)NULL;
-    time = ((t1 << 16) & 0xff0000) | ((t2 << 8) & 0xff00) | ((t3 << 16) & 0xff000000) | (t3 & 0xff);
-    token = MakeToken(time, seqnum, class, (TOKEN *)NULL);
+    now = ((t1 << 16) & 0xff0000) | ((t2 << 8) & 0xff00) | ((t3 << 16) & 0xff000000) | (t3 & 0xff);
+    token = MakeToken(now, seqnum, class, (TOKEN *)NULL);
     return &token;
 }
 
-BOOL timehash_init(BOOL *selfexpire) {
-    *selfexpire = FALSE;
+bool timehash_init(SMATTRIBUTE *attr) {
+    if (attr == NULL) {
+	syslog(L_ERROR, "timehash: attr is NULL");
+	SMseterror(SMERR_INTERNAL, "attr is NULL");
+	return false;
+    }
+    attr->selfexpire = false;
+    attr->expensivestat = true;
     if (STORAGE_TOKEN_LENGTH < 6) {
 	syslog(L_FATAL, "timehash: token length is less than 6 bytes");
 	SMseterror(SMERR_TOKENSHORT, NULL);
-	return FALSE;
+	return false;
     }
-    return TRUE;
+    return true;
 }
 
 TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
@@ -110,7 +118,7 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     time_t              now;
     TOKEN               token;
     int                 fd;
-    int                 result;
+    ssize_t             result;
     int                 seq;
     int                 i;
 
@@ -129,10 +137,10 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
 		continue;
 	    p = strrchr(path, '/');
 	    *p = '\0';
-	    if (!MakeDirectory(path, TRUE)) {
+	    if (!MakeDirectory(path, true)) {
 	        syslog(L_ERROR, "timehash: could not make directory %s %m", path);
 	        token.type = TOKEN_EMPTY;
-	        DISPOSE(path);
+	        free(path);
 	        SMseterror(SMERR_UNDEFINED, NULL);
 	        return token;
 	    } else {
@@ -141,7 +149,7 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
 		    SMseterror(SMERR_UNDEFINED, NULL);
 		    syslog(L_ERROR, "timehash: could not open %s %m", path);
 		    token.type = TOKEN_EMPTY;
-		    DISPOSE(path);
+		    free(path);
 		    return token;
 	        }
 	    }
@@ -150,23 +158,24 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     }
     if (i == 0x10000) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	syslog(L_ERROR, "timehash: all sequence numbers for the time and class are reserved %d %d", now, class);
+	syslog(L_ERROR, "timehash: all sequence numbers for the time and class are reserved %lu %d", (unsigned long)now, class);
 	token.type = TOKEN_EMPTY;
-	DISPOSE(path);
+	free(path);
 	return token;
     }
 
-    if ((result = write(fd, article.data, article.len)) != article.len) {
+    result = xwritev(fd, article.iov, article.iovcnt);
+    if (result != (ssize_t) article.len) {
 	SMseterror(SMERR_UNDEFINED, NULL);
 	syslog(L_ERROR, "timehash error writing %s %m", path);
 	close(fd);
 	token.type = TOKEN_EMPTY;
 	unlink(path);
-	DISPOSE(path);
+	free(path);
 	return token;
     }
     close(fd);
-    DISPOSE(path);
+    free(path);
     return MakeToken(now, seq, class, article.token);
 }
 
@@ -177,48 +186,57 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
     struct stat         sb;
     ARTHANDLE           *art;
 
+    if (amount == RETR_STAT) {
+        if (access(path, R_OK) < 0) {
+            SMseterror(SMERR_UNDEFINED, NULL);
+            return NULL;
+        }
+        art = xmalloc(sizeof(ARTHANDLE));
+        art->type = TOKEN_TIMEHASH;
+	art->data = NULL;
+	art->len = 0;
+	art->private = NULL;
+	return art;
+    }
+
     if ((fd = open(path, O_RDONLY)) < 0) {
 	SMseterror(SMERR_UNDEFINED, NULL);
 	return NULL;
     }
 
-    art = NEW(ARTHANDLE, 1);
+    art = xmalloc(sizeof(ARTHANDLE));
     art->type = TOKEN_TIMEHASH;
 
-    if (amount == RETR_STAT) {
-	art->data = NULL;
-	art->len = 0;
-	art->private = NULL;
-	close(fd);
-	return art;
-    }
-
     if (fstat(fd, &sb) < 0) {
 	SMseterror(SMERR_UNDEFINED, NULL);
 	syslog(L_ERROR, "timehash: could not fstat article: %m");
-	DISPOSE(art);
+	free(art);
 	return NULL;
     }
     
-    private = NEW(PRIV_TIMEHASH, 1);
+    private = xmalloc(sizeof(PRIV_TIMEHASH));
     art->private = (void *)private;
     private->len = sb.st_size;
     if (innconf->articlemmap) {
-	if ((private->base = mmap((MMAP_PTR)0, sb.st_size, PROT_READ, MAP__ARG, fd, 0)) == (MMAP_PTR)-1) {
+	if ((private->base = mmap(NULL, sb.st_size, PROT_READ, MAP_SHARED, fd, 0)) == MAP_FAILED) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
 	    syslog(L_ERROR, "timehash: could not mmap article: %m");
-	    DISPOSE(art->private);
-	    DISPOSE(art);
+	    free(art->private);
+	    free(art);
 	    return NULL;
 	}
+        if (amount == RETR_ALL)
+            madvise(private->base, sb.st_size, MADV_WILLNEED);
+        else
+            madvise(private->base, sb.st_size, MADV_SEQUENTIAL);
     } else {
-	private->base = NEW(char, private->len);
+	private->base = xmalloc(private->len);
 	if (read(fd, private->base, private->len) < 0) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
 	    syslog(L_ERROR, "timehash: could not read article: %m");
-	    DISPOSE(private->base);
-	    DISPOSE(art->private);
-	    DISPOSE(art);
+	    free(private->base);
+	    free(art->private);
+	    free(art);
 	    return NULL;
 	}
     }
@@ -238,10 +256,14 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
 	return art;
     }
     
-    if ((p = SMFindBody(private->base, private->len)) == NULL) {
+    if ((p = wire_findbody(private->base, private->len)) == NULL) {
 	SMseterror(SMERR_NOBODY, NULL);
-	DISPOSE(art->private);
-	DISPOSE(art);
+	if (innconf->articlemmap)
+	    munmap(private->base, private->len);
+	else
+	    free(private->base);
+	free(art->private);
+	free(art);
 	return NULL;
     }
 
@@ -257,13 +279,17 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
 	return art;
     }
     SMseterror(SMERR_UNDEFINED, "Invalid retrieve request");
-    DISPOSE(art->private);
-    DISPOSE(art);
+    if (innconf->articlemmap)
+	munmap(private->base, private->len);
+    else
+	free(private->base);
+    free(art->private);
+    free(art);
     return NULL;
 }
 
 ARTHANDLE *timehash_retrieve(const TOKEN token, const RETRTYPE amount) {
-    int                 time;
+    int                 now;
     int                 seqnum;
     char                *path;
     ARTHANDLE           *art;
@@ -274,14 +300,14 @@ ARTHANDLE *timehash_retrieve(const TOKEN token, const RETRTYPE amount) {
 	return NULL;
     }
 
-    BreakToken(token, &time, &seqnum);
-    path = MakePath(time, seqnum, token.class);
+    BreakToken(token, &now, &seqnum);
+    path = MakePath(now, seqnum, token.class);
     if ((art = OpenArticle(path, amount)) != (ARTHANDLE *)NULL) {
-	art->arrived = time;
+	art->arrived = now;
 	ret_token = token;
 	art->token = &ret_token;
     }
-    DISPOSE(path);
+    free(path);
     return art;
 }
 
@@ -293,14 +319,10 @@ void timehash_freearticle(ARTHANDLE *article) {
     
     if (article->private) {
 	private = (PRIV_TIMEHASH *)article->private;
-	if (innconf->articlemmap) {
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-	    madvise(private->base, private->len, MADV_DONTNEED);
-#endif
+	if (innconf->articlemmap)
 	    munmap(private->base, private->len);
-	} else {
-	    DISPOSE(private->base);
-	}
+	else
+	    free(private->base);
 	if (private->top)
 	    closedir(private->top);
 	if (private->sec)
@@ -309,26 +331,26 @@ void timehash_freearticle(ARTHANDLE *article) {
 	    closedir(private->ter);
 	if (private->artdir)
 	    closedir(private->artdir);
-	DISPOSE(private);
+	free(private);
     }
-    DISPOSE(article);
+    free(article);
 }
 
-BOOL timehash_cancel(TOKEN token) {
-    int                 time;
+bool timehash_cancel(TOKEN token) {
+    int                 now;
     int                 seqnum;
     char                *path;
     int                 result;
 
-    BreakToken(token, &time, &seqnum);
-    path = MakePath(time, seqnum, token.class);
+    BreakToken(token, &now, &seqnum);
+    path = MakePath(now, seqnum, token.class);
     result = unlink(path);
-    DISPOSE(path);
+    free(path);
     if (result < 0) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	return FALSE;
+	return false;
     }
-    return TRUE;
+    return true;
 }
 
 static struct dirent *FindDir(DIR *dir, FINDTYPE type) {
@@ -370,8 +392,10 @@ ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
     struct dirent       *de;
     ARTHANDLE           *art;
     int                 seqnum;
+    size_t              length;
 
-    path = NEW(char, strlen(innconf->patharticles) + 32);
+    length = strlen(innconf->patharticles) + 32;
+    path = xmalloc(length);
     if (article == NULL) {
 	priv.top = NULL;
 	priv.sec = NULL;
@@ -382,15 +406,13 @@ ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
 	priv.terde = NULL;
     } else {
 	priv = *(PRIV_TIMEHASH *)article->private;
-	DISPOSE(article->private);
-	DISPOSE(article);
-	if (innconf->articlemmap) {
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-	    madvise(priv.base, priv.len, MADV_DONTNEED);
-#endif
-	    munmap(priv.base, priv.len);
-	} else {
-	    DISPOSE(priv.base);
+	free(article->private);
+	free((void *)article);
+	if (priv.base != NULL) {
+	    if (innconf->articlemmap)
+		munmap(priv.base, priv.len);
+	    else
+		free(priv.base);
 	}
     }
 
@@ -414,45 +436,47 @@ ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
 			/* end of search */
 			closedir(priv.top);
 			priv.top = NULL;
-			DISPOSE(path);
+			free(path);
 			return NULL;
 		    }
-		    sprintf(path, "%s", innconf->patharticles);
+		    snprintf(path, length, "%s", innconf->patharticles);
 		    if ((priv.top = opendir(path)) == NULL) {
 			SMseterror(SMERR_UNDEFINED, NULL);
-			DISPOSE(path);
+			free(path);
 			return NULL;
 		    }
 		    if ((priv.topde = FindDir(priv.top, FIND_TOPDIR)) == NULL) {
 			SMseterror(SMERR_UNDEFINED, NULL);
 			closedir(priv.top);
-			DISPOSE(path);
+			free(path);
 			return NULL;
 		    }
 		}
-		sprintf(path, "%s/%s", innconf->patharticles, priv.topde->d_name);
+		snprintf(path, length, "%s/%s", innconf->patharticles, priv.topde->d_name);
 		if ((priv.sec = opendir(path)) == NULL)
 		    continue;
 	    }
-	    sprintf(path, "%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name);
+	    snprintf(path, length, "%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name);
 	    if ((priv.ter = opendir(path)) == NULL)
 		continue;
 	}
-	sprintf(path, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
+	snprintf(path, length, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
 	if ((priv.artdir = opendir(path)) == NULL)
 	    continue;
     }
     if (de == NULL)
 	return NULL;
-    sprintf(path, "%s/%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
+    snprintf(path, length, "%s/%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
 
     art = OpenArticle(path, amount);
     if (art == (ARTHANDLE *)NULL) {
-	art = NEW(ARTHANDLE, 1);
+	art = xmalloc(sizeof(ARTHANDLE));
 	art->type = TOKEN_TIMEHASH;
 	art->data = NULL;
 	art->len = 0;
-	art->private = (void *)NEW(PRIV_TIMEHASH, 1);
+	art->private = xmalloc(sizeof(PRIV_TIMEHASH));
+	newpriv = (PRIV_TIMEHASH *)art->private;
+	newpriv->base = NULL;
     }
     newpriv = (PRIV_TIMEHASH *)art->private;
     newpriv->top = priv.top;
@@ -462,12 +486,45 @@ ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
     newpriv->topde = priv.topde;
     newpriv->secde = priv.secde;
     newpriv->terde = priv.terde;
-    sprintf(path, "%s/%s/%s/%s", priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
+    snprintf(path, length, "%s/%s/%s/%s", priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
     art->token = PathToToken(path);
     BreakToken(*art->token, (int *)&(art->arrived), &seqnum);
-    DISPOSE(path);
+    free(path);
     return art;
 }
 
+bool timehash_ctl(PROBETYPE type, TOKEN *token UNUSED, void *value) {
+    struct artngnum *ann;
+
+    switch (type) {
+    case SMARTNGNUM:
+	if ((ann = (struct artngnum *)value) == NULL)
+	    return false;
+	/* make SMprobe() call timehash_retrieve() */
+	ann->artnum = 0;
+	return true;
+    default:
+	return false;
+    }
+}
+
+bool
+timehash_flushcacheddata(FLUSHTYPE type UNUSED)
+{
+    return true;
+}
+
+void
+timehash_printfiles(FILE *file, TOKEN token, char **xref UNUSED,
+                    int ngroups UNUSED)
+{
+    int now, seqnum;
+    char *path;
+    
+    BreakToken(token, &now, &seqnum);
+    path = MakePath(now, seqnum, token.class);
+    fprintf(file, "%s\n", path);
+}
+
 void timehash_shutdown(void) {
 }
